/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "../libs.h"
#include "../file/assetmanager.h"
#include "../engine/engine.h"
#include "../display/renderlist.h"
#include "../display/shader.h"

// Forwarder for scene_t
typedef struct _scene_t scene_t;

// Callbacks
typedef void scenecallback_t(scene_t *scene);

// Scene Definition
typedef struct _scene_t {
  engine_t *engine;
  assetmanagerowner_t assetOwner;
  void *user;

  /** Whether or not the assets that this scene depends upon have loaded */
  bool loaded;

  /** Scene Callbacks */
  scenecallback_t *onLoaded;
  scenecallback_t *onRender;
} scene_t;

void sceneInit(scene_t *scene, engine_t *engine, void *user);

float sceneUpdate(scene_t *scene);

void sceneDispose(scene_t *scene);